@@ -1,5 +1,5 @@
 ## A pre-view function for tscs data
-## 2018-3-10
+## 2018-6-24
 
 ## raw: plot raw data
 ## missing: plot treatment status 
@@ -8,11 +8,12 @@
 ## preview of data treatment status, missing values and raw data ##
 ##---------------------------------------------------------------##
 
-panelView <- function(formula,
-                      data, # a data frame (long-form)
+panelView <- function(data, # a data frame (long-form)
+                      formula,
                       index, # c(unit, time) indicators
-                      na.rm = FALSE, # remove missing values
-                      outcome.type = "c", # c or d: continuous or discrete
+                      na.rm = TRUE, # remove missing values
+                      treatment = TRUE,
+                      outcome.type = "continuous", # continuous or discrete
                       type = "missing",
                       by.group = FALSE, # (color pre-treatment treated differently)
                       theme.bw = FALSE,
@@ -23,66 +24,70 @@ panelView <- function(formula,
                       legendOff = FALSE,
                       legend.labs = NULL,
                       main = NULL,
+                      by.treatment = FALSE,
                       id = NULL,
                       show.id = NULL,
-                      raw.color = NULL,
+                      color = NULL,
                       axis.adjust = FALSE,
                       axis.lab = "both",
                       axis.lab.gap = c(0, 0)
                     ) {  
-    ## ------------------------------------- ##
-    ##          part 1: parsing data
-    ## ------------------------------------- ##
+        
+    ## ------------------------------- ##
+    ## part1. parsing data             ##
+    ## ------------------------------- ##
+
+    ## ------------------------- ##
+    ## parse variable.           ##
+    ## ------------------------- ##
+
     varnames <- all.vars(formula)
     Y <- varnames[1]
-    D <- varnames[2]
-    if (length(varnames) > 2) {
-        X <- varnames[3:length(varnames)]
+
+    if (length(varnames) == 1) { ## only y
+        D <- X <- NULL
+        treatment <- 0
+    } else if (length(varnames) == 2) {
+        if (treatment == 1) {
+            D <- varnames[2]
+            X <- NULL
+        } else {
+            D <- NULL
+            X <- varnames[2]
+        }   
     } else {
-        X <- NULL
-    }
-
-    p <- length(X)
-
-    namesData <- names(data)
-    for (i in 1:length(varnames)) {
-        if(!varnames[i] %in% namesData) {
-            stop(paste("variable \"", varnames[i],"\" is not in the data set.", sep = ""))
+        if (treatment == 1) {
+            D <- varnames[2]
+            X <- varnames[3:length(varnames)]
+        } else {
+            D <- NULL
+            X <- varnames[2:length(varnames)]
         }
     }
 
-    ##store variable names
-    ## id.old <- id
-    if (is.null(id)) {
-        id.old <- NULL
-    } else {
-        id.old <- sort(id)
+    if (is.data.frame(data) == FALSE || length(class(data)) > 1) {
+        data <- as.data.frame(data)
+        warning("Not a data frame.")
     }
-    data.old <- data
-    Yname <- Y
-    Dname <- D
-    Xname <- X
-    
-    id <- index[1]
-    time <- index[2]
 
-    TT <- length(unique(data[,time]))
-    N <- length(unique(data[,id]))
-    id.series <- unique(sort(data[,id])) ## unit id
-    time.uni <- unique(sort(data[,time])) ## period
+    data <- data[,c(index, Y, D, X)] ## covariates are excluded
+    if (na.rm == FALSE & sum(is.na(data)) > 0) {
+        stop("Missing values in dataset. Try set na.rm = TRUE.\n")
+    } 
+    if (na.rm == TRUE) {
+        data <- na.omit(data)
+    } 
+    
+    ## p <- length(X)
+
     ##-------------------------------##
     ## Checking Parameters
     ##-------------------------------## 
-    ## library(ggplot2)
 
     if (!type %in% c("missing", "raw")) {
         stop("\"type\" option misspecified.")
     }
 
-    if (is.data.frame(data) == FALSE) {
-        data <- as.data.frame(data)
-        warning("Not a data frame.")
-    }
     ## index
     if (length(index) != 2 | sum(index %in% colnames(data)) != 2) {
         stop("\"index\" option misspecified. Try, for example, index = c(\"unit.id\", \"time\").")
@@ -90,7 +95,7 @@ panelView <- function(formula,
     
     unique_label <- unique(paste(data[,index[1]],"_",data[,index[2]],sep=""))
     if (length(unique_label)!=dim(data)[1]) {
-        stop("Some records may be duplicated or wrongly marked in the data set.")
+        stop("Unit and time variables do not uniquely identify all observations. Some may be duplicated or wrongly marked in the dataset.")
     }
 
     ## remove missing values
@@ -112,6 +117,72 @@ panelView <- function(formula,
         legend.labs <- as.character(legend.labs)
     }
 
+    ## outcome.type
+    if (!outcome.type %in% c("continuous", "discrete")) {
+        stop("\"outcome.type\" option misspecified. Try, for example, outcome.type = c(\"continuous\", \"discrete\").")
+    }
+
+    ## treatment indicator
+    if (treatment == 1) {
+        if (!(1%in%data[, D] & 0%in%data[, D] & length(unique(data[,D])) == 2)) {
+            stop(paste("variable \"", D, "\" should contain only 0 and 1.\n"))
+        }
+    }
+
+    ## plot
+    raw.color <- NULL
+
+    ## ------------------------ ##
+    ## parsing data.            ##
+    ## ------------------------ ##
+
+    namesData <- names(data)
+    for (i in 1:length(varnames)) {
+        if(!varnames[i] %in% namesData) {
+            stop(paste("variable \"", varnames[i],"\" is not in the data set.", sep = ""))
+        }
+    }
+
+    ## raw id 
+    id.series.old <- unique(data[,index[1]])
+
+    ##store variable names
+    ## specified id 
+    if (is.null(id)) {
+        id.old <- NULL
+    } else {
+        id.old <- sort(id)
+    }
+
+    data.old <- data
+    Yname <- Y
+    Dname <- D
+    Xname <- X
+    
+    id <- index[1]
+    time <- index[2]
+
+    ## if (class(data[,id])=="factor") {
+    ##     data[,id] <- as.character(data[,id])
+    ## }
+
+    ## if (class(data[,time])=="factor") {
+    ##     data[,time] <- as.character(data[,time])
+    ## }    
+
+    TT <- length(unique(data[,time]))
+    N <- length(unique(data[,id]))
+    id.series <- unique(sort(data[,id])) ## unit id
+    time.uni <- unique(sort(data[,time])) ## period
+
+    remove.id <- NULL
+
+    if (length(id.series.old) > length(id.series)) {
+        remove.id <- setdiff(id.series.old, id.series)
+        cat("list of removed units from dataset:", remove.id)
+        cat("\n\n")
+    }
+
     ## missing plot : y-axis
     if (!is.null(id.old)) {
         if (!is.null(show.id)) {
@@ -120,8 +191,19 @@ panelView <- function(formula,
         if (length(unique(id.old)) != length(id.old)) {
             stop("Repeated values in \"id\" option.")
         }
+
+        if (sum(id.old%in%id.series.old) < length(id.old)) {
+            id.old.err <- which(!id.old%in%id.series.old)
+            cat("Some specified units will be removed due to missingness:", id.old[id.old.err])
+            cat("\n\n")  
+            id.old <- id.old[-id.old.err]
+        }
+
         if (sum(id.old%in%id.series) < length(id.old)) {
-            stop("Some specified units are not in the data.\n")    
+            id.old.rm <- which(id.old %in% remove.id)
+            cat("list of removed units from \"id\":", id.old[id.old.rm], sep = " ")
+            cat("\n\n")
+            id.old <- id.old[-id.old.rm]
         } 
         ## else {
         ##     id.series <- sort(id.old)
@@ -143,75 +225,67 @@ panelView <- function(formula,
                 stop("Repeated values in \"show.id\" option.")
             }
             id.old <- id.series[sort(show.id)] 
+        } else {
+            id.old <- id.series
         }
     }
 
-    ##-------------------------------##
-    ## Parsing raw data
-    ##-------------------------------##  
+    if (length(id.old) != length(id.series)) {
+        data.old <- data <- data[which(data[,id] %in% id.old),]
+        N <- length(id.old)
+    }
 
     ## sort data
     data <- data[order(data[,id], data[,time]), ]
     id.all <- time.all <- count <- coordin <- data.x <- x.na <- NULL
-    
-    ## if (p > 0) { ## covariates may contain missing values
-        ## location
-    ##     id.all <- as.matrix(data[, id])
-    ##     time.all <- as.matrix(data[, time])
 
-    ##     count <- dim(data)[1]
-    ##     coordin <- matrix(NA, count, 2)
-    ##     for (i in 1:count) {
-    ## 	    coordin[i, 2] <- which(id.series == id.all[i])
-    ##	    coordin[i, 1] <- which(time.uni == time.all[i]) ## obs.missing: T*N
-    ##     }
-    ##     data.x <- as.matrix(data[, X])
-    ##     x.na <- apply(is.na(data.x), 1, sum)
-    ##     x.na <- ifelse(x.na > 0, 1, 0)
-    ## }
-
-    if (na.rm == TRUE) {
-        data <- data[,c(index, Y, D, X)] ## covariates are excluded
-        data <- na.omit(data)
-    } 
+    ## check treatment indicator
+    ## treatment <- 1
+    if (treatment == 1 & is.null(Dname)) {
+        warning("No treatment indicator.\n")
+        treatment <- 0    
+    }
 
     ## check missingness
-    if (sum(is.na(data[, Yname])) > 0) {
-        stop(paste("Missing values in variable \"", Yname,"\".", sep = ""))
-    }
-    if (sum(is.na(data[, Dname])) > 0) {
-        stop(paste("Missing values in variable \"", Dname,"\".", sep = ""))
-    }
-
-    if (!(1%in%data[, Dname]&0%in%data[,Dname]&length(unique(data[,Dname]))==2)) {
-        stop(paste("Error values in variable \"", Dname,"\".", sep = ""))
-    }
-
-    if (sum(is.na(data[, id])) > 0) {
-        stop(paste("Missing values in variable \"", id,"\".", sep = ""))
-    }
-    if (sum(is.na(data[, time])) > 0) {
-        stop(paste("Missing values in variable \"", time,"\".", sep = ""))
-    } 
+    ## if (sum(is.na(data[, Yname])) > 0) {
+    ##     stop(paste("Missing values in variable \"", Yname,"\".", sep = ""))
+    ## }
+    ## if (treatment ==1 & sum(is.na(data[, Dname])) > 0) {
+    ##     stop(paste("Missing values in variable \"", Dname,"\".", sep = ""))
+    ## }
+    ## if (sum(is.na(data[, id])) > 0) {
+    ##     stop(paste("Missing values in variable \"", id,"\".", sep = ""))
+    ## }
+    ## if (sum(is.na(data[, time])) > 0) {
+    ##     stop(paste("Missing values in variable \"", time,"\".", sep = ""))
+    ## } 
 
     ## check balanced panel and fill unbalanced panel
-    if (var(table(data[,id])) + var(table(data[, time])) > 0 | TT == N) {
+    Y <- I <- D <- NULL
+    if (dim(data)[1] != TT*N) {
 
         data[,id] <- as.numeric(as.factor(data[,id]))
         data[,time] <- as.numeric(as.factor(data[,time]))
 
         Y <- matrix(NA, TT, N)
-        I <- D <- matrix(0, TT, N)
+        I <- matrix(0, TT, N)
+        if (treatment == 1) {
+            D <- matrix(0, TT, N)
+        }
         for (i in 1:dim(data)[1]) {
             Y[data[i,time],data[i,id]] <- data[i,Yname]
-            D[data[i,time],data[i,id]] <- data[i,Dname]
+            if (treatment == 1) {
+                D[data[i,time],data[i,id]] <- data[i,Dname]
+            }
             I[data[i,time],data[i,id]] <- 1
         }
 
     } else {
         I <- matrix(1, TT, N)
         Y <- matrix(data[,Yname], TT, N)
-        D <- matrix(data[,Dname], TT, N)
+        if (treatment == 1) {
+            D <- matrix(data[,Dname], TT, N)
+        }
     }
 
     ## index matrix that indicates if data is observed 
@@ -225,131 +299,165 @@ panelView <- function(formula,
     
     ##treatment indicator
     ## D <- matrix(data[,Dname],TT,N)
-    D.old <- D ## store D 
+    
+    if (treatment == 1) {
+        D.old <- D ## store D 
 
-    ## once treated, always treated
-    D <- apply(D, 2, function(vec){cumsum(vec)})
-    D <- ifelse(D > 0, 1, 0)
+        ## once treated, always treated
+        D <- apply(D, 2, function(vec){cumsum(vec)})
+        D <- ifelse(D > 0, 1, 0)
 
-    ## DID timing
-    tr.old <- D[TT,]==1
-    D.tr.old <- as.matrix(D[,which(tr.old==1)])
-    T0 <- apply(D.tr.old==0,2,sum) 
-    DID <- length(unique(T0))==1
+        ## DID timing
+        tr.old <- D[TT,]==1
+        D.tr.old <- as.matrix(D[,which(tr.old==1)])
+        T0 <- apply(D.tr.old==0,2,sum) 
+        DID <- length(unique(T0))==1
 
-    ## check DID mode
-    if (sum(abs(D.old[which(I==1)] - D[which(I==1)])) == 0) {
-        by.group <- by.group
-        FEmode <- 0
-    } else { ## FE mode
-        DID <- 0
-        if (by.group == FALSE & type == "raw") {
-            warning("Treatment has reversals.\n")
+        ## check DID mode
+        if (sum(abs(D.old[which(I==1)] - D[which(I==1)])) == 0) {
+            by.group <- by.group
+            FEmode <- 0
+        } else { ## FE mode
+            DID <- 0
+            if (by.group == FALSE & type == "raw") {
+                warning("Treatment has reversals.\n")
+            }
+            ## by.group <- TRUE
+            FEmode <- 1
         }
-        ## by.group <- TRUE
-        FEmode <- 1
     }
 
     ## raw plot color setting
-    if (is.null(raw.color)) {
-        if (theme.bw == FALSE) {
-            raw.color <- c("#99999950", "#FC8D6280", "red")
-        } else {
-            if (outcome.type == "c") {
-                raw.color <- c("#5e5e5e50", "#4671D565", "#06266F")
+    if (type == "raw") {
+        if (is.null(color)) {
+            if (theme.bw == FALSE) {
+                if (treatment == 1) {
+                    raw.color <- c("#99999950", "#FC8D6280", "red")
+                } else {
+                    raw.color <- "#99999950"
+                }
             } else {
-                raw.color <- c("#5e5e5e60", "#4671D5", "#06266F")
+                if (outcome.type == "continuous") {
+                    if (treatment == 1) {
+                        raw.color <- c("#5e5e5e50", "#4671D565", "#06266F")
+                    } else {
+                        raw.color <- "#5e5e5e50"
+                    }
+                } else {
+                    if (treatment == 1) {
+                        raw.color <- c("#5e5e5e60", "#4671D5", "#06266F")
+                    } else {
+                        raw.color <- "#5e5e5e60"
+                    }
+                }
+            }
+        } else {        
+            if (treatment == 1) {
+                if (FEmode == 1) {
+                    if (length(color) != 2) {
+                        stop("Length of \"color\" should be equal to 2.\n")    
+                    } else {
+                        cat("Specified colors are in the order of \"under treatment\", \"under control\".\n")
+                        raw.color <- color[c(2,1)]
+                    }
+                } else {
+                    if (by.group == FALSE & length(color) != 3) {
+                        stop("Length of \"color\" should be equal to 3.\n")
+                    }
+                    else if (by.group == TRUE & length(color) != 2) {
+                        stop("Length of \"color\" should be equal to 2.\n")
+                    }
+                    else if (by.group == FALSE & length(color) == 3) {
+                        cat("Specified colors in the order of \"treated (pre)\", \"treated (post)\", \"control\".\n")
+                        raw.color <- color[c(3,1,2)]
+                    }
+                    else {
+                        cat("Specified colors in the order of \"under treatment\", \"under control\".\n")
+                        raw.color <- color[c(2,1)]
+                    }
+                }
+            } else {
+                if (length(color) != 1) {
+                   stop("Length of \"raw.color\" should be equal to 1.\n") 
+                }
             }
         }
-    } else {
-        
-        if (FEmode == 1) {
-            if (length(raw.color) != 2) {
-                stop("Length of \"raw.color\" shold be equal to 2.\n")    
-            }
-        } else {
-            if (by.group == FALSE & length(raw.color) != 3) {
-                stop("Length of \"raw.color\" shold be equal to 3.\n")
-            }
-            else if (by.group == TRUE & length(raw.color) != 2) {
-                stop("Length of \"raw.color\" shold be equal to 2.\n")
-            }
-        }
-    }
+    } 
 
     ##-------------------------------##
     ## storage
     ##-------------------------------##    
-    ## if (is.null(id.old)) {
-        iname <- unique(sort(data.old[,id]))
-    ## } else {
-    ##     iname <- sort(id.old)
-    ## }
+    iname <- unique(sort(data.old[,id]))
     tname <- unique(sort(data.old[,time]))
 
-    ## outcome variable
-    ## Y <- matrix(data[,Yname],TT,N)
-    ## Y[which(I==0)] <- NA
-
     ## plot a part of outcomes: raw plot
-    if (!is.null(id.old)) {
-        id.pos <- rep(NA, length(id.old))
-        for (i in 1:length(id.old)) {
-            if (id.old[i]%in%iname) {
-                id.pos[i] <- which(iname == id.old[i])
-            } else {
-                stop("Some specified units are not in the data.")
-            }
-        }
-        N <- length(id.old)
-        Y <- as.matrix(Y[, id.pos])
-        I <- as.matrix(I[, id.pos])
-        D <- as.matrix(D[, id.pos])
-        D.old <- as.matrix(D.old[, id.pos])
-        iname <- id.old
-    }
+    ## if (!is.null(id.old)) {
+    ##     id.pos <- rep(NA, length(id.old))
+    ##     for (i in 1:length(id.old)) {
+    ##         if (id.old[i]%in%iname) {
+    ##             id.pos[i] <- which(iname == id.old[i])
+    ##         } else {
+    ##             stop("Some specified units are not in the data.")
+    ##         }
+    ##     }
+    ##     N <- length(id.old)
+    ##     Y <- as.matrix(Y[, id.pos])
+    ##     I <- as.matrix(I[, id.pos])
+    ##     if (treatment == 1) {
+    ##         D <- as.matrix(D[, id.pos])
+    ##         D.old <- as.matrix(D.old[, id.pos])
+    ##     }
+    ##     iname <- id.old
+    ## }
 
-    if (FEmode == 0 && by.group == FALSE) {
-        tr <- D[TT,]==1     # cross-sectional: treated unit
-        pre <- as.matrix(D[,which(tr==1)]==0&I[,which(tr==1)]==1) # a matrix indicating before treatment
-        post <- as.matrix(D[,which(tr==1)]==1&I[,which(tr==1)]==1)
-        id.tr <- which(tr==1)
-        id.co <- which(tr==0)
+    ## missing matrix 
+    obs.missing <- NULL
+    if (treatment == 1) {
+        if (FEmode == 0 && by.group == FALSE) {
+            tr <- D[TT,]==1     # cross-sectional: treated unit
+            pre <- as.matrix(D[,which(tr==1)]==0&I[,which(tr==1)]==1) # a matrix indicating before treatment
+            post <- as.matrix(D[,which(tr==1)]==1&I[,which(tr==1)]==1)
+            id.tr <- which(tr==1)
+            id.co <- which(tr==0)
 
-        D.tr <- as.matrix(D[,which(tr==1)])
-        I.tr <- as.matrix(I[,which(tr==1)])
-        Y.tr <- as.matrix(Y[,which(tr==1)])
-        Y.co <- as.matrix(Y[,which(tr==0)])
+            D.tr <- as.matrix(D[,which(tr==1)])
+            I.tr <- as.matrix(I[,which(tr==1)])
+            Y.tr <- as.matrix(Y[,which(tr==1)])
+            Y.co <- as.matrix(Y[,which(tr==0)])
 
-        Ntr <- sum(tr)
-        Nco <- N - Ntr
+            Ntr <- sum(tr)
+            Nco <- N - Ntr
 
-        obs.missing <- matrix(3, TT, N) ## control group:1
-        pre[which(pre==1)] <- 1 ## pre 2
-        post[which(post==1)] <- 2 ## post 3
-        obs.missing[,id.tr] <- pre + post
-        obs.missing[which(I==0)] <- 4 ## missing
+            obs.missing <- matrix(3, TT, N) ## control group:1
+            pre[which(pre==1)] <- 1 ## pre 2
+            post[which(post==1)] <- 2 ## post 3
+            obs.missing[,id.tr] <- pre + post
+            obs.missing[which(I==0)] <- 4 ## missing
 
-        id.tr <- iname[which(tr==1)] ## re-define id
-        id.co <- iname[which(tr==0)]
-    } else {
-        unit.type <- rep(NA, N) ## 1 for control; 2 for treated; 3 for reversal
-        for (i in 1:N) {
-            di <- D.old[, i]
-            ii <- I[, i]
-            if (length(unique(di[which(ii==1)])) == 1) { ## treated or control
-                if (0 %in% unique(di[which(ii==1)])) {
-                    unit.type[i] <- 1 ## control
+            id.tr <- iname[which(tr==1)] ## re-define id
+            id.co <- iname[which(tr==0)]
+        } else {
+            unit.type <- rep(NA, N) ## 1 for control; 2 for treated; 3 for reversal
+            for (i in 1:N) {
+                di <- D.old[, i]
+                ii <- I[, i]
+                if (length(unique(di[which(ii==1)])) == 1) { ## treated or control
+                    if (0 %in% unique(di[which(ii==1)])) {
+                        unit.type[i] <- 1 ## control
+                    } else {
+                        unit.type[i] <- 2 ## treated
+                    }
                 } else {
-                    unit.type[i] <- 2 ## treated
+                    unit.type[i] <- 3 ## reversal
                 }
-            } else {
-                unit.type[i] <- 3 ## reversal
             }
-        }
         
-        obs.missing <- matrix(2, TT, N) ## not under treatment
-        obs.missing[which(D.old==1)] <- 1 ## under treatment
+            obs.missing <- matrix(2, TT, N) ## not under treatment
+            obs.missing[which(D.old==1)] <- 1 ## under treatment
+            obs.missing[which(I==0)] <- 4 ## missing
+        }
+    } else {
+        obs.missing <- matrix(2, TT, N) ## observed 
         obs.missing[which(I==0)] <- 4 ## missing
     }
 
@@ -422,7 +530,7 @@ panelView <- function(formula,
             stop("Some element in \"xlim\" is not numeric.")
         } else {
             if (length(xlim)!=2) {
-                stop("xlim must be of length 2.")
+                stop("\"xlim\" must be of length 2.")
             }
         }
     }
@@ -431,7 +539,7 @@ panelView <- function(formula,
             stop("Some element in \"ylim\" is not numeric.")
         } else {
             if (length(ylim)!=2) {
-                stop("ylim must be of length 2.")
+                stop("\"ylim\" must be of length 2.")
             }
         }
     }
@@ -477,21 +585,21 @@ panelView <- function(formula,
         }
     }
     
-    if (type == "missing") {
-        if (is.null(id) == TRUE) {
+    ## if (type == "missing") {
+    ##     if (is.null(id) == TRUE) {
             ## if (is.null(show.id) == TRUE) {
-                id <- colnames(obs.missing)
+    ##             id <- colnames(obs.missing)
             ## } else {
             ##     id <- colnames(obs.missing)[show.id]
             ## }
-        }
+    ##     }
         ## m.l <- length(id)
         ## for (i in 1:m.l) {
         ##     if (!id[i]%in%colnames(obs.missing)) {
         ##         stop("Some specified units are not in the data.")
         ##     }
         ## }
-    }
+    ## }
   
     ## type of plots
     if (!is.numeric(time[1])) {
@@ -518,7 +626,6 @@ panelView <- function(formula,
 
     ############  START  ############### 
     if (type == "raw") {
-
         ## axes labels
         if (is.null(xlab)==TRUE) {
             xlab <- index[2]
@@ -531,10 +638,140 @@ panelView <- function(formula,
             ylab <- NULL
         }
 
-        if (by.group == FALSE) { ## Mixed units
+        if (treatment == 0) {
+            if (outcome.type == "continuous") {
+                data <- cbind.data.frame("time" = rep(time[show], N),
+                                         "outcome" = c(Y[show,]),
+                                         "type" = rep("co",(N*nT)),
+                                         "id" = rep(1:N,each = nT))
+            } else {
+                data <- cbind.data.frame("time" = rep(time[show], N), 
+                                         "outcome" = factor(c(Y[show,])),
+                                         "type" = factor(rep("co",(N*nT))))
+                ## remove NA
+                if (sum(is.na(data[,"outcome"])) > 0) {
+                    data <- data[-which(is.na(data[,"outcome"])),]
+                }
+            }
+
+            ## theme
+            p <- ggplot(data) + xlab(xlab) +  ylab(ylab)
+
+            if (theme.bw == FALSE) {
+                p <- p + theme(legend.position = legend.pos,
+                               axis.text.x = element_text(angle = angle, hjust=x.h, vjust=x.h),
+                               plot.title = element_text(size=20,
+                                                         hjust = 0.5,
+                                                         face="bold",
+                                                         margin = margin(10, 0, 10, 0)))
+            } else {
+                p <- p + theme_bw()
+                p <- p + theme(panel.grid.major = element_blank(),
+                               panel.grid.minor = element_blank(),
+                               legend.position = legend.pos,
+                               axis.text.x = element_text(angle = angle, hjust=x.h, vjust=x.h),
+                                plot.title = element_text(size=20,
+                                                          hjust = 0.5,
+                                                          face="bold",
+                                                          margin = margin(10, 0, 10, 0)))
+            }
+
+            if (outcome.type == "continuous") {
+                ## main
+                p <- p + geom_line(aes(time, outcome,
+                                       colour = type,
+                                       size = type,
+                                       linetype = type,
+                                       group = id))
+
+                ## legend
+                set.limits = "co"
+                set.colors = raw.color
+                set.linetypes = "solid"
+                set.linewidth = 0.5
+                if (!is.null(legend.labs)) {
+                    if (length(legend.labs) != 1) {
+                        warning("Wrong number of labels in the legend. Using default.\n")
+                        set.labels = "Observed"  
+                    } else {
+                        set.labels <- legend.labs
+                    }
+                } else {
+                    set.labels <- "Observed"
+                }
+                labels.ncol <- 1
+
+                ## scale
+                p <- p + scale_colour_manual(limits = set.limits,
+                                             labels = set.labels,
+                                             values =set.colors) +
+                    scale_linetype_manual(limits = set.limits,
+                                          labels = set.labels,
+                                          values = set.linetypes) +
+                    scale_size_manual(limits = set.limits,
+                                      labels = set.labels,
+                                      values = set.linewidth) +
+                    guides(linetype = guide_legend(title=NULL, ncol=labels.ncol),
+                           colour = guide_legend(title=NULL, ncol=labels.ncol),
+                           size = guide_legend(title=NULL, ncol=labels.ncol))
+
+            } else { ## categorical data
+              
+                ## main
+                p <- p + geom_jitter(width = 0.15, height = 0.15,
+                                     aes(x = time, y = outcome, colour = type, shape = type))
+
+                ## scale
+                set.limits = "co"
+                set.colors = raw.color
+                set.shapes = 1
+                if (!is.null(legend.labs)) {
+                    if (length(legend.labs) != 1) {
+                        warning("Wrong number of labels in the legend. Using default.\n")
+                        set.labels = "Observed"
+                    } else {
+                        set.labels <- legend.labs
+                    }
+                } else {                      
+                    set.labels = "Observed"
+                }
+              
+                labels.ncol <- 1
+
+                p <- p + scale_colour_manual(limits = set.limits,
+                                             labels = set.labels,
+                                             values =set.colors) + 
+                         scale_shape_manual(limits = set.limits,
+                                            labels = set.labels,
+                                            values =set.shapes) + 
+                         guides(colour = guide_legend(title=NULL, ncol=labels.ncol),
+                                shape = guide_legend(title=NULL, ncol=labels.ncol))
+            }
+            if (!is.numeric(time.label)) {
+                p <- p + 
+                     scale_x_continuous(expand = c(0, 0), breaks = show[T.b], labels = time.label[T.b])
+            }
+
+            ## title
+            if (is.null(main) == TRUE) {
+                p <- p + ggtitle("Raw Data")
+            } else if (main!="") {
+                p <- p + ggtitle(main)
+            }
+
+            ## ylim
+            if (is.null(ylim) == FALSE) {
+                p <- p + coord_cartesian(ylim = ylim)
+            }
+            
+            suppressWarnings(print(p))
+
+        }
+
+        else if (treatment == 1 && by.group == FALSE) { ## Mixed units
             
             ## time-line
-            if (outcome.type == "c") { ## continuous outcome
+            if (outcome.type == "continuous") { ## continuous outcome
                 if (FEmode == 0) { ## DID data
                     ## if (length(id) == 1) {
                     ##     time.bf <- time[T0[which(id == id.tr)]]
@@ -559,6 +796,7 @@ panelView <- function(formula,
                                                         rep("co",(Nco*nT)),
                                                         rep("tr.pst",length(Y.tr.pst))),
                                              "id" = c(rep(1:N,each = nT), id.tr.pst*(-1)))
+
                 } else { ## FE mode data
                 
                     D.plot <- D.old
@@ -648,12 +886,12 @@ panelView <- function(formula,
                     if (!is.null(legend.labs)) {
                         if (length(legend.labs) != 2) {
                             warning("Wrong number of labels in the legend. Using default.\n")
-                            set.labels = c("Control", "Treatment")  
+                            set.labels = c("Control", "Treated")  
                         } else {
                             set.labels <- legend.labs
                         }
                     } else {
-                        set.labels <- c("Control", "Treatment") 
+                        set.labels <- c("Control", "Treated") 
                     }
                     labels.ncol <- 2
                 }            
@@ -810,7 +1048,7 @@ panelView <- function(formula,
             if (1 %in% unit.type) {
                 co.pos <- which(unit.type == 1)
                 Nco <- length(co.pos)
-                if (outcome.type == "c") {
+                if (outcome.type == "continuous") {
                     data1 <- cbind.data.frame("time" = c(rep(time[show], Nco)),
                                               "outcome" = c(Y[show, co.pos]),
                                               "type" = c(rep("co", (Nco*nT))),
@@ -820,6 +1058,10 @@ panelView <- function(formula,
                                               "outcome" = factor(c(Y[show, co.pos])),
                                               "type" = factor(c(rep("co", (Nco*nT)))),
                                               "id" = c(rep(1:Nco, each = nT)))
+                    ## remove NA
+                    if (sum(is.na(data1[,"outcome"])) > 0) {
+                        data1 <- data1[-which(is.na(data1[,"outcome"])),]
+                    }
 
                 }
                 limits1 <- c("co", "tr")
@@ -833,7 +1075,7 @@ panelView <- function(formula,
             if (2 %in% unit.type) {
                 tr.pos <- which(unit.type == 2)
                 Ntr <- length(tr.pos)
-                if (outcome.type == "c") {
+                if (outcome.type == "continuous") {
                     data2 <- cbind.data.frame("time" = c(rep(time[show], Ntr)),
                                               "outcome" = c(Y[show, tr.pos]),
                                               "type" = c(rep("tr",(Ntr*nT))),
@@ -843,6 +1085,12 @@ panelView <- function(formula,
                                               "outcome" = factor(c(Y[show, tr.pos])),
                                               "type" = factor(c(rep("tr",(Ntr*nT)))),
                                               "id" = c(rep(1:Ntr,each = nT)))
+
+                    ## remove NA
+                    if (sum(is.na(data2[,"outcome"])) > 0) {
+                        data2 <- data2[-which(is.na(data2[,"outcome"])),]
+                    }
+
                 }
                 limits2 <- c("co", "tr")
                 #limits2 <- "tr"
@@ -883,7 +1131,7 @@ panelView <- function(formula,
                 }
 
 
-                if (outcome.type == "c") {
+                if (outcome.type == "continuous") {
                     data3 <- cbind.data.frame("time" = c(rep(time[show], Nrv), ut.time),
                                               "outcome" = c(c(Y[show, rv.pos]),
                                                           c(Y.trt.show[which(!is.na(Y.trt.show))])),
@@ -897,6 +1145,22 @@ panelView <- function(formula,
                                               "type" = factor(c(rep("co",(Nrv*nT)),
                                                        rep("tr",length(ut.id)))),
                                               "id" = c(rep(1:Nrv,each = nT), ut.id))
+
+                    data3 <- cbind.data.frame("time" = c(rep(time[show], Nrv)),
+                                              "outcome" = factor(c(c(Y[show, rv.pos]))),
+                                              "type2" = factor(c(obs.missing[show, rv.pos])),
+                                              "id" = c(rep(1:Nrv,each = nT)))
+
+                    type <- rep(NA, dim(data3)[1])
+                    type[which(data3[,"type2"] == 2)] <- "co"
+                    type[which(data3[,"type2"] == 1)] <- "tr"
+                    data3[,"type"] <- type
+
+                    ## remove NA
+                    if (sum(is.na(data3[,"outcome"])) > 0) {
+                        data3 <- data3[-which(is.na(data3[,"outcome"])),]
+                    }
+
                 }
                 limits3 <- c("co", "tr")
                 colors3 <- raw.color[1:2]
@@ -935,7 +1199,7 @@ panelView <- function(formula,
                 }
 
                 ## main
-                if (outcome.type == "c") {
+                if (outcome.type == "continuous") {
                     p <- p + geom_line(aes(time, outcome,
                                            colour = type,
                                            size = type,
@@ -1091,9 +1355,7 @@ panelView <- function(formula,
                                          top = textGrob(main, gp = gpar(fontsize=20,font=2))))
                 }
             }
-            ## end of raw plot
-        }
-        
+        }    ## end of raw plot
     } else if (type=="missing") {
         
         if (is.null(xlab)==TRUE) {
@@ -1107,14 +1369,18 @@ panelView <- function(formula,
             ylab <- NULL
         }
         if (is.null(main)==TRUE) {
-            main <- "Treatment Status"
+            if (treatment == 1) {
+                main <- "Treatment Status"
+            } else {
+                main <- "Missing Values"
+            }
         } else if (main == "") {
             main <- NULL
         }
 
         m <- obs.missing
         
-        if (!is.null(id)) {
+        if (length(id) < dim(m)[2]) {
             m <- as.matrix(m[show,which(colnames(m)%in%id)])
         } else {
             ## if (!is.null(show.id)) {
@@ -1142,6 +1408,7 @@ panelView <- function(formula,
                 breaks <- c(breaks,3)
                 label <- c(label,"Controls")
             }
+
         } else {
             if (1%in%all) {
                 col <- c(col,"#06266F")
@@ -1151,7 +1418,11 @@ panelView <- function(formula,
             if (2%in%all) {
                 col <- c(col,"#B0C4DE")
                 breaks <- c(breaks,2)
-                label <- c(label,"Under Control")
+                if (treatment == 1) {
+                    label <- c(label,"Under Control")
+                } else {
+                    label <- c(label, "Observed")
+                }
             }
         }
         if (4%in%all) {
@@ -1159,6 +1430,28 @@ panelView <- function(formula,
             breaks <- c(breaks,4)
             label <- c(label,"Missing")
         }
+        if (!is.null(color)) {
+            if (length(col) == length(color)) {
+                if (treatment == 1) { ## with treatment indicator
+                    if (FEmode == 0) {
+                        if (length(col) == 3) {
+                            cat("Specified colors are in order of \"treated (pre)\", \"treated (post)\", \"control\".\n")
+                        } else {
+                            cat("Specified colors are in order of \"treated (pre)\", \"treated (post)\", \"control\", \"missing\".\n")
+                        }
+                    } else {
+                        if (length(col) == 2) {
+                            cat("Specified colors are in order of \"treatment\", \"control\".\n")
+                        } else {
+                            cat("Specified colors are in order of \"treatment\", \"control\", \"missing\".\n")
+                        }
+                    }
+                }
+                col <- color
+            } else {
+                stop(paste("Length of \"color\" shold be equal to ",length(col),".\n", sep=""))
+            }
+        }
         if (!is.null(legend.labs)) {
             if (length(legend.labs) != length(all)) {
                 warning("Wrong number of labels in the legend. Using default.\n")
@@ -1170,6 +1463,12 @@ panelView <- function(formula,
         N <- dim(m)[2]
         units <- rep(rev(1:N), each = TT)
         period <- rep(1:TT, N)
+        if (3 %in% all && by.treatment == TRUE) {
+            missing.tr <- which(apply(m == 3, 2, sum) > 0)
+            missing.co <- which(apply(m == 3, 2, sum) == 0)
+            m <- m[, c(missing.co, missing.tr)]
+            id <- id[c(missing.co, missing.tr)]
+        }
         res <- c(m)
         data <- cbind.data.frame(units=units, period=period, res=res)
         data[,"res"] <- as.factor(data[,"res"])
@@ -1233,6 +1532,7 @@ panelView <- function(formula,
                                         hjust = 0.5,
                                         face="bold",
                                         margin = margin(10, 0, 10, 0)))
+
         if (axis.lab == "both") {
             p <- p + scale_x_continuous(expand = c(0, 0), breaks = T.b, labels = time.label[T.b]) +
             scale_y_continuous(expand = c(0, 0), breaks = N.b, labels = id[N.b])
@@ -1261,5 +1561,5 @@ panelView <- function(formula,
     ## } else {
     ##     out <- list(plot = p)
     ## }
-    ## return(out)   
+    ## return(p)   
 }
\ No newline at end of file
